      subroutine dnulp2_a(lda,na,a,b,c,lpindx,nlpindx,
     &                    jj,x,dinv,v,info)

*     This program is to solve
*                    min c^Tx
*                 a_i^T x <= b_i, for all i with i=lpindx(j), j=1,...,nlpindx
*     with initial feasible point x, its correponding index set jj of the base,
*     and the inverse matrix dinv of the base given, but jj(i) may be zero.
*     The rank of the coefficient matrix A may not be full. If the rank of 
*     A is r, the program will exit ONLY when r constraints reach equality.
*     On exit, info = r for full rank case and info = -r otherwise.
 
      implicit none
 
*  .. Scalar Arguments ..
      integer          lda, na, nlpindx, info
 
*  .. Array Arguments ..
      integer          jj(*), lpindx(*)
      double precision a(lda,*), b(*), c(*), x(*), dinv(lda,*), v(*)

*  .. Input

*     lda,na,a,b,c,lpindx,nlpindx,jj,x,dinv:
*        lda: leading dimension of a, >= na+1
*          a: each column is the coefficients of a constraint
*             its active part is a(1:na,1:lpindx(nlpindx))
*          b: constant terms, b(1:1:lpindx(nlpindx)) is active


*  .. Output

*          x: a nondegenerate feasible solution
*         jj: the indices of the base constraints
*       dinv: the inverse of a base matrix whose rows are the coefficients
*             of the base constraints
*       info: = rank for full rank case; = -rank otherwise

 
*  .. Workspace
      integer          ell, k, k_, i, i1, j, flag
      double precision eps, sigj, vmax, bot, top, dtmp

*  .. Executable
 
      eps = 1.0d-6

***** step 1                               To find which constraint to kick out
 
 10   do i = 1, na
         if(jj(i) .eq. 0) goto 20
      enddo
      goto 40

 20   vmax = -1.0d0
      do i = 1, na
         if(jj(i) .eq. 0) then
            dtmp = 0.0d0
            do j = 1, na
               dtmp = dtmp + c(j)*dinv(j,i)
            enddo
            if (abs(dtmp) .gt. vmax) then
               bot = dtmp
               vmax = abs(bot)
               k = i
            endif
         endif
      enddo

      if(vmax .gt. eps) then
         if(bot .ge. 0.0d0) then
            do i = 1, na
               v(i) = dinv(i,k)
            enddo
         else
            do i = 1, na
               v(i) = -dinv(i,k)
            enddo
         endif
         goto 100
      endif
 
 40   vmax = -1.0d150
      do i = 1, na
         if (jj(i) .ne. 0) then
            dtmp = 0.0d0
            do j = 1, na
               dtmp = dtmp + c(j)*dinv(j,i)
            enddo
            if (dtmp .gt. vmax) then
               vmax = dtmp
               k = i
            endif
         endif
      enddo

      if(vmax .lt. eps) then
         goto 500                                   ! with an optimal solution
      endif

      do i = 1, na
         v(i) = dinv(i,k)
      enddo
 
***** step 2                                To find which constraint coming in
 
 100  sigj = 1.0d150  

      ell = 0
      do i1 = 1, nlpindx
         i = lpindx(i1)
         do j = 1, na
            if (jj(j) .eq. i) goto 110
         enddo
         bot = 0.0d0
         do j = 1, na
            bot = bot + a(j,i)*v(j)      
         enddo
         if (bot .ge. -eps) goto 110
         top = -b(i)
         do j = 1, na
            top = top + a(j,i)*x(j)
         enddo
         top = top/bot
         if (top .ge. sigj) goto 110
         sigj = top
         ell = i
 110  enddo

      if (ell .eq. 0) then
         print*, 'ell=0, wrong in dnulp2_a.f'
         stop                             ! unbounded, never happen in our case
      endif

***** step 3                                              To update x, jj, dinv
 
      do i = 1, na
         x(i) = x(i) - sigj*v(i)
      enddo
      do i = 1, na
         v(i) = a(i,ell)
      enddo
 
*     to find the new B^{-1}
 
      top = 0.0d0
      do i = 1, na
         top = top + v(i)*dinv(i,k)
      enddo
      if (abs(top) .lt. eps) then
         write(*,*) 'singular in dnulp2_a.f. sum = ', top
         stop
      endif
 
      top = 1.0d0/top
      do i = 1, na
         dinv(i,k) = dinv(i,k)*top
      enddo
      do j = 1, na
         if (j.ne.k) then
            top = 0.0d0
            do i = 1, na
               top = top + v(i)*dinv(i,j)
            enddo
            do i = 1, na
               dinv(i,j) = dinv(i,j) - top*dinv(i,k)
            enddo
         endif
      enddo
 
      jj(k) = ell

      goto 10

 500  k_ = 0
      info = 0
      do i = 1, na
         if (jj(i) .gt. 0) then
             info = info + 1
         endif
      enddo

      if (info .eq. na) return

      k = 1
      do while (k .le. na)
 
         if (jj(k) .gt. 0 .or. k .eq.  k_) then
            k = k + 1
            goto 700
         endif

         do i = 1, na
            v(i) = dinv(i,k)
         enddo
         flag = 1
 
*****    do the step 2                       To find which constraint coming in
 
 600     sigj = 1.0d150  

         ell = 0
         do i1 = 1, nlpindx
            i = lpindx(i1)
            do j = 1, na
               if (jj(j) .eq. i) goto 610
            enddo
            bot = 0.0d0
            do j = 1, na
               bot = bot + a(j,i)*v(j)      
            enddo
            if (bot .ge. -eps) goto 610
            top = -b(i)
            do j = 1, na
               top = top + a(j,i)*x(j)
            enddo
            top = top/bot
            if (top .ge. sigj) goto 610
            sigj = top
            ell = i
 610     enddo

         if (ell .eq. 0) then 
            if (flag .eq. 1) then
               do i = 1, na
                  v(i) = -dinv(i,k)
               enddo
               flag = 0
               goto 600
            else
               k = k + 1
               goto 700
            endif
         endif

*****    do the step 3                   Found an ell. Then update x, jj, dinv
 
         do i = 1, na
            x(i) = x(i) - sigj*v(i)
         enddo
         do i = 1, na
            v(i) = a(i,ell)
         enddo
 
*        to find the new B^{-1}
 
         top = 0.0d0
         do i = 1, na
            top = top + v(i)*dinv(i,k)
         enddo
         if (abs(top) .lt. eps) then
            write(*,*) 'singular in dnulp2_a.f. sum = ', top
            stop
         endif
 
         top = 1.0d0/top
         do i = 1, na
            dinv(i,k) = dinv(i,k)*top
         enddo
         do j = 1, na
            if (j.ne.k) then
               top = 0.0d0
               do i = 1, na
                  top = top + v(i)*dinv(i,j)
               enddo
               do i = 1, na
                  dinv(i,j) = dinv(i,j) - top*dinv(i,k)
               enddo
            endif
         enddo
 
         jj(k) = ell
         info = info + 1

         if (info .eq. na) then
            return
         else
            k_ = k
            k = 1                                                 ! necessary?
         endif

 700  enddo

      info = - info                                ! the case of not full rank

      return
      end
