      subroutine dnulp2_e(lda,na_,ma_,a,b,lpindx,nlpindx_,
     &                    jj,x,dinv,v,info)

      implicit none

*     This program is to find a nondegenerate solution for
*                 a_i^T x <= b_i, for all i with i=lpindx(j), j=1,...,nlpindx_
*     by solving     min x(na_+1)
*                    a_i^T x            <= b_i  for all i with b_i nonnegative 
*                    a_i^T x - x(na_+1) <= b_i  for all i with b_i negative 
*                             -x(na_+1) <= 0 
*     We assume that the LP problem is feasible (This is always true in our
*     case. This program will exit ONLY when rank-of-A inequalities in the 
*     constrants reach equality (and x(na_+1)=0). 
 
*  .. Scalar Arguments ..
      integer          lda, na_, ma_, nlpindx_, info
 
*  .. Array Arguments ..
      integer          jj(*), lpindx(*)
      double precision a(lda,*), b(*), x(*), dinv(lda,*), v(*)

*  .. Input

* lda,na_,ma_,a,b,lpindx,nlpindx_,jj,x,dinv:
*        lda: leading dimension of a, >= na_+1
*          a: each column is the coefficients of a constraint
*             its active part is a(1:na_,1:ma_)
*          b: constant terms, b(1:ma_) is active

*  .. Output

*          x: a nondegenerate feasible solution
*         jj: the indices of the base constraints
*       dinv: the inverse of a base matrix whose rows are the coefficients
*             of the base constraints
*       info: = rank for full rank case; = -rank otherwise

 
*  .. Workspace
      integer          ell, k, k_, i, i1, j, na, ma, nlpindx, flag
      double precision eps, sigj, vmax, bot, top, dtmp

*  .. Executable
 
      eps = 1.0d-6

***** Expand the system (add variable epsilon)

      na = na_ + 1
      ma = ma_ + 1
      nlpindx = nlpindx_ + 1
      dtmp = 0.0d0
      ell = ma                            ! the index of the equality equation
      do i = 1, nlpindx_
         j = lpindx(i)
         if (b(j) .lt. -eps) then
            a(na,j) = -1
            if (-b(j) .gt. dtmp) then
               dtmp = -b(j)
               ell = j
            endif
         else
            a(na,j) = 0
         endif
      enddo
      do i = 1, na_
         a(i,ma) = 0
      enddo
      a(na,ma) = -1
      b(ma) = 0
      lpindx(nlpindx) = ma

***** To form the first x, jj, dinv

      do i = 1, na_
         x(i) = 0.0d0
         jj(i) = 0
      enddo
      x(na) = dtmp
      jj(na) = ell
      do j = 1, na
         do i = 1, na
            dinv(i,j) = 0.0d0
         enddo
         dinv(j,j) = 1.0d0
      enddo
      do j = 1, na_
         dinv(na,j) = a(j,ell)
      enddo
      dinv(na,na) = -1.0d0
 
***** To apply the LP algorithm
***** step 1                              To find which constraint to kick out
 
 10   do i = 1, na
         if(jj(i) .eq. 0) goto 20
      enddo
      goto 40

 20   vmax = -1.0d0
      do i = 1, na
         if(jj(i) .eq. 0 ) then
            dtmp = dinv(na,i)
            if (abs(dtmp) .gt. vmax) then
               bot = dtmp
               vmax = abs(bot)
               k = i
            endif
         endif
      enddo

      if(vmax .gt. eps) then
         if(bot .ge. 0.0d0) then
            do i = 1, na
               v(i) = dinv(i,k)
            enddo
         else
            do i = 1, na
               v(i) = -dinv(i,k)
            enddo
         endif
         goto 100
      endif
 
 40   vmax = -1.0d150
      do i = 1, na
         if (jj(i) .ne. 0) then
            dtmp = dinv(na,i)
            if (dtmp .gt. vmax) then
               vmax = dtmp
               k = i
            endif
         endif
      enddo

      if(vmax .lt. eps) then
         goto 200                                   ! found an optimal solution
      endif

      do i = 1, na
         v(i) = dinv(i,k)
      enddo
 
***** step 2                                 To find which constraint coming in
 
 100  sigj = 1.0d150  

      ell = 0
      do i1 = 1, nlpindx
         i = lpindx(i1)
         do j = 1, na
            if (jj(j) .eq. i) goto 110
         enddo
         bot = 0.0d0
         do j = 1, na
            bot = bot + a(j,i)*v(j)
         enddo
         if (bot .ge. -eps) goto 110
         top = -b(i)
         do j = 1, na
            top = top + a(j,i)*x(j)
         enddo
         top = top/bot
         if (top .ge. sigj) goto 110
         sigj = top
         ell = i
 110  enddo

      if (ell .eq. 0) then
         print*, 'Unbounded, wrong in dnulp2_e.f'
         stop                             ! unbounded, never happen in our case
      endif

***** step 3                                              To update x, jj, dinv
 
      do i = 1, na
         x(i) = x(i) - sigj*v(i)
      enddo
      do i = 1, na
         v(i) = a(i,ell)
      enddo
 
*     to find the new B^{-1}
 
      top = 0.0d0
      do i = 1, na
         top = top + v(i)*dinv(i,k)
      enddo
      if (abs(top) .lt. eps) then
         write(*,*) 'singular in dnulp2_e.f. sum = ', top
         stop
      endif
 
      top = 1.0d0/top
      do i = 1, na
         dinv(i,k) = dinv(i,k)*top
      enddo
      do j = 1, na
         if (j.ne.k) then
            top = 0.0d0
            do i = 1, na
               top = top + v(i)*dinv(i,j)
            enddo
            do i = 1, na
               dinv(i,j) = dinv(i,j) - top*dinv(i,k)
            enddo
         endif
      enddo
 
      jj(k) = ell

      goto 10

***** To change x, jj, dinv into the ones without the x(na)=epsilon variable

 200  if (jj(na) .eq. ma) goto 250

      k = 0
      j = 0
      do while (j .lt. na)
         j = j + 1
         if (jj(j) .eq. ma) then
            k = j
            j = na
         endif
      enddo

      if (k .eq. 0) then                                          ! for testing
         print*, 'no index ma in jj, wrong in dnulp2_e.f'
         stop                                          ! not happen in our case
      endif

      jj(k) = jj(na)
      do i = 1, na_
         dinv(i,k) = dinv(i,na)
      enddo

 250  do i = 1, nlpindx_                          ! reset the value WRT epsilon
         a(na,lpindx(i)) = 0.0d0
      enddo 

      k_ = 0
 500  info = 0
      do i = 1, na_
         if (jj(i) .gt. 0) then
             info = info + 1
         endif
      enddo

      if (info .eq. na_) return

      k = 1
      do while (k .le. na_)
 
         if (jj(k) .gt. 0 .or. k .eq.  k_) then
            k = k + 1
            goto 700
         endif

         do i = 1, na_
            v(i) = dinv(i,k)
         enddo
         flag = 1
 
*****    do the step 2                       To find which constraint coming in
 
 600     sigj = 1.0d150  

         ell = 0
         do i1 = 1, nlpindx_
            i = lpindx(i1)
            do j = 1, na_
               if (jj(j) .eq. i) goto 610
            enddo
            bot = 0.0d0
            do j = 1, na_
               bot = bot + a(j,i)*v(j)      
            enddo
            if (bot .ge. -eps) goto 610
            top = -b(i)
            do j = 1, na_
               top = top + a(j,i)*x(j)
            enddo
            top = top/bot
            if (top .ge. sigj) goto 610
            sigj = top
            ell = i
 610     enddo

         if (ell .eq. 0) then 
            if (flag .eq. 1) then
               do i = 1, na_
                  v(i) = -dinv(i,k)
               enddo
               flag = 0
               goto 600
            else
               k = k + 1
               goto 700
            endif
         endif

*****    do the step 3                   Found an ell. Then update x, jj, dinv
 
         do i = 1, na_
            x(i) = x(i) - sigj*v(i)
         enddo
         do i = 1, na_
            v(i) = a(i,ell)
         enddo
 
*        to find the new B^{-1}
 
         top = 0.0d0
         do i = 1, na_
            top = top + v(i)*dinv(i,k)
         enddo
         if (abs(top) .lt. eps) then
            write(*,*) 'singular in dnulp2_e.f. sum = ', top
            stop
         endif
 
         top = 1.0d0/top
         do i = 1, na_
            dinv(i,k) = dinv(i,k)*top
         enddo
         do j = 1, na_
            if (j.ne.k) then
               top = 0.0d0
               do i = 1, na_
                  top = top + v(i)*dinv(i,j)
               enddo
               do i = 1, na_
                  dinv(i,j) = dinv(i,j) - top*dinv(i,k)
               enddo
            endif
         enddo
 
         jj(k) = ell
         info = info + 1

         if (info .eq. na_) then
            return
         else
            k_ = k
            k = 1                                                  ! necessary?
         endif

 700  enddo

      info = - info                                ! the case of not full rank

      return
      end
